@@ -17,6 +17,7 @@ from cloudmesh.shell.command import PluginCommand
 from cloudmesh.shell.command import command
 from cloudmesh.shell.command import map_parameters
 # from cloudmesh.common.debug import VERBOSE
+from cloudmesh.burn.Imager import Imager
 
 
 class BurnCommand(PluginCommand):
@@ -32,6 +33,7 @@ class BurnCommand(PluginCommand):
               burn install
               burn load --device=DEVICE
               burn format --device=DEVICE
+              burn imager [TAG...]
               burn mount [--device=DEVICE] [--os=OS]
               burn unmount [--device=DEVICE] [--os=OS]
               burn network list [--ip=IP] [--used]
@@ -44,6 +46,10 @@ class BurnCommand(PluginCommand):
               burn backup [--device=DEVICE] [--to=DESTINATION]
               burn copy [--device=DEVICE] [--from=DESTINATION]
               burn shrink [--image=IMAGE]
+              burn cluster --device=DEVICE --hostname=HOSTNAME
+                           [--ip=IP]
+                           [--ssid=SSID]
+                           [--wifipassword=PSK]
               burn create [--image=IMAGE]
                           [--device=DEVICE]
                           [--hostname=HOSTNAME]
@@ -56,6 +62,8 @@ class BurnCommand(PluginCommand):
                           [--wifipassword=PSK]
                           [--format]
                           [--tag=TAG]
+                          [--inventory=INVENTORY]
+                          [--name=NAME]
               burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
               burn set [--hostname=HOSTNAME]
                        [--ip=IP]
@@ -70,7 +78,7 @@ class BurnCommand(PluginCommand):
               --version              Show version.
               --image=IMAGE          The image filename,
                                      e.g. 2019-09-26-raspbian-buster.img
-              --device=DEVICE        The device, e.g. /dev/mmcblk0
+              --device=DEVICE        The device, e.g. /dev/sdX
               --hostname=HOSTNAME    The hostname
               --ip=IP                The IP address
               --key=KEY              The name of the SSH key file
@@ -85,13 +93,19 @@ class BurnCommand(PluginCommand):
                 Location where the images will be stored for reuse
 
             Description:
+                cms burn create --inventory=INVENTORY --device=DEVICE --name=NAME
+
+                    Will refer to a specified cloudmesh inventory file (see cms help inventory).
+                    Will search the configurations for NAME inside of INVENTORY and will burn
+                    to DEVICE. Supports parameter expansion.
+
                 cms burn create --passwd=PASSWD
 
                      if the passwd flag is added the default password is
                      queried from the commandline and added to all SDCards
 
-                     if the flag is ommitted login via the password is disabled
-                     and only login via the sshkey is allowed
+                     if the flag is omitted login via the password is
+                     disabled and only login via the sshkey is allowed
 
               Network
 
@@ -139,19 +153,21 @@ class BurnCommand(PluginCommand):
 
                 cms burn install
 
-                    installs a program to shring img files. THis is useful, after
-                    you created a backup to make the backup smaller and allow
-                    faster burning in case of recovery
+                    installs a program to shrink img files. THis is
+                    useful, after you created a backup to make the
+                    backup smaller and allow faster burning in case of
+                    recovery
 
                 cms burn load --device=DEVICE
 
                     loads the sdcard into the USB drive. Thi sis similar to
-                    loading a cdrom drive. It s the oposite to eject
+                    loading a cdrom drive. It s the opposite to eject
 
                 cms burn format --device=DEVICE
 
-                    formats the SDCard in the specified device. Be careful it is
-                    the correct device.  cms burn info will help you to identifying it
+                    formats the SDCard in the specified device. Be
+                    careful it is the correct device.  cms burn info
+                    will help you to identifying it
 
                 cms burn mount [--device=DEVICE] [--os=OS]
 
@@ -167,23 +183,29 @@ class BurnCommand(PluginCommand):
 
                 cms burn image versions [--refresh] [--yaml]
 
-                    The images that you like to burn onto your SDCard can be cached locally with the image command.
-                    The available images for the PI can be found when using the --refresh option. If you do not
-                    specify it it reads a copy of the image list from our cache
+                    The images that you like to burn onto your SDCard
+                    can be cached locally with the image command.  The
+                    available images for the PI can be found when
+                    using the --refresh option. If you do not specify
+                    it it reads a copy of the image list from our
+                    cache
 
                 cms burn image ls
 
-                    Lists all downloaded images in our cache. You can download
-                    them with the cms burn image get command
+                    Lists all downloaded images in our cache. You can
+                    download them with the cms burn image get command
 
                 cms burn image delete [--image=IMAGE]
 
-                    deletes the specified image. The name can be found with the image ls command
+                    deletes the specified image. The name can be found
+                    with the image ls command
 
                 cms burn image get [--url=URL] [TAG...]
 
-                    downloads a specific image or the latest image. The tag are a number of words
-                    separated by a space that must occur in the tag that you find in the versions command
+                    downloads a specific image or the latest
+                    image. The tag are a number of words separated by
+                    a space that must occur in the tag that you find
+                    in the versions command
 
                 cms burn backup [--device=DEVICE] [--to=DESTINATION]
 
@@ -192,12 +214,14 @@ class BurnCommand(PluginCommand):
                 cms burn copy [--device=DEVICE] [--from=DESTINATION]
 
                     copies the file form the destination on the SDCard
-                    this is the same as the SDCard command. we will in future remove one
+                    this is the same as the SDCard command. we will in
+                    future remove one
 
                 cms burn shrink [--image=IMAGE]
 
-                    shrinks the size of a backoup or image file that is on
-                    your local file system. It can only be used for .img files
+                    shrinks the size of a backup or image file that
+                    is on your local file system. It can only be used
+                    for .img files
 
                 cms burn create [--image=IMAGE]
                                 [--device=DEVICE]
@@ -211,9 +235,8 @@ class BurnCommand(PluginCommand):
                                 [--wifipassword=PSK]
                                 [--format]
 
-                    This is a comprehensif cuntion that not only can format the SDCard, but also
-                    initializes it with specific falues
-
+                    This command  not only can format the SDCard, but
+                    also initializes it with specific values
 
                 cms burn sdcard [TAG...] [--device=DEVICE] [--dryrun]
 
@@ -224,8 +247,9 @@ class BurnCommand(PluginCommand):
                              [--key=KEY]
                              [--mount=MOUNTPOINT]
 
-                    this sets specific values on the sdcard after it has ben created
-                    with the creat, copy or sdcard command
+                    this sets specific values on the sdcard after it
+                    has ben created with the create, copy or sdcard
+                    command
 
                     a --ssh is missing from this command
 
@@ -235,18 +259,19 @@ class BurnCommand(PluginCommand):
 
                 cms burn wifi --ssid=SSID [--passwd=PASSWD] [--country=COUNTRY]
 
-                    this sets the wifi ssid and password afterthe card is created,
-                    copies, or sdcard is used.
+                    this sets the wifi ssid and password after the card
+                    is created, copied, or the sdcard is used.
 
-                    The option country option expects an ISO
-                    3166-1 two digit country code. The default is "US" and
+                    The option country option expects an ISO 3166-1
+                    two digit country code. The default is "US" and
                     the option not required if suitable. See
                     https://en.wikipedia.org/wiki/ISO_3166-1 for other
                     countries.
 
                 cms burn check [--device=DEVICE]
 
-                    this command lists the parameters that were set with the set or create command
+                    this command lists the parameters that were set
+                    with the set or create command
 
             Examples: ( \\ is not shown)
 
@@ -274,6 +299,7 @@ class BurnCommand(PluginCommand):
                        "sshkey",
                        "blocksize",
                        "dryrun",
+                       # "output",
                        "ssid",
                        "url",
                        "key",
@@ -282,11 +308,14 @@ class BurnCommand(PluginCommand):
                        "version",
                        "to",
                        "os",
-                       "country")
+                       "country",
+                       "inventory",
+                       "name")
         # arguments.MOUNTPOINT = arguments["--mount"]
         arguments.FORMAT = arguments["--format"]
         arguments.FROM = arguments["--from"]
         arguments.IMAGE = arguments["--image"]
+        arguments.output = "table"  # hard code for now
 
         # VERBOSE(arguments)
 
@@ -304,7 +333,24 @@ class BurnCommand(PluginCommand):
         StopWatch.stop("info")
         StopWatch.status("info", True)
 
-        if arguments.firmware and arguments.check:
+        if arguments.imager:
+
+            arguments.TAG = arguments.TAG or ["latest-lite"]
+
+            Console.msg(f"Tags: {arguments.TAG}")
+            try:
+                file = Imager.fetch(tag=arguments.TAG)
+            except:
+                pass
+
+            try:
+                Imager.launch(file=file)
+            except Exception as e:
+                Console.error(f"could not find image with the tag {arguments.TAG}\n\n{e}\n")
+
+            return ""
+
+        elif arguments.firmware and arguments.check:
 
             execute("firmware check", burner.firmware(action="check"))
             return ""
@@ -346,7 +392,7 @@ class BurnCommand(PluginCommand):
             return ""
 
         elif arguments["format"]:  # as format is a python word, we need to use an index
-            execute("format", burner.format_device(device=arguments.device))
+            execute("format", burner.format_device(device=arguments.device, unmount=True))
             return ""
 
         elif arguments.network and arguments["list"]:
@@ -407,7 +453,8 @@ class BurnCommand(PluginCommand):
 
         elif arguments.info:
 
-            execute("info", burner.info())
+            output = arguments.output or "table"
+            execute("info", burner.info(output=output))
             return ""
 
         elif arguments.install:
@@ -486,6 +533,11 @@ class BurnCommand(PluginCommand):
             return ""
 
         elif arguments["get"] and arguments['image'] and arguments["TAG"]:
+
+            tag = arguments["TAG"]
+            if "latest" in tag and ("full" in tag or "lite" in tag):
+                result = Image.create_version_cache(refresh=arguments["--refresh"])
+
             image = Image()
             execute("image fetch", image.fetch(tag=arguments["TAG"]))
             return ""
@@ -495,6 +547,41 @@ class BurnCommand(PluginCommand):
             execute("image fetch", image.fetch(tag="latest"))
             return ""
 
+        elif arguments.cluster:
+
+            # is true when
+            #
+            # cms burn cluster --hostname=red,red00[1-2]
+            #                  --device=/dev/sdb
+            #                  --ip=10.1.1.[1-3]
+            #                  --ssid=myssid
+            #                  --wifipassword=mypass
+            #
+
+            execute("cluster", burner.cluster(arguments=arguments))
+            return ""
+
+        elif arguments.create and arguments.inventory:
+            if not os_is_pi():
+                Console.error("This command has only been safely tested on Raspberry Pis. Terminating for caution")
+                return
+            if not arguments.name:
+                Console.error("Missing --name parameter. See cms help burn for usage")
+                return ""
+            if not arguments.device:
+                Console.error("Missing --device parameter. See cms help burn for usage")
+                return ""
+
+            multi_burner = MultiBurner()
+            # Perhaps we want to change the path at some point
+            inventory = f"~/.cloudmesh/{arguments.inventory}"
+            execute("burn inventory",
+                    multi_burner.burn_inventory(
+                        inventory=inventory,
+                        name=arguments.name,
+                        device=arguments.device))
+            return ""
+
         elif arguments.create:
 
             if arguments["--passwd"]:
@@ -502,7 +589,6 @@ class BurnCommand(PluginCommand):
             elif "PASSWD" in os.environ:
                 passwd = os.environ["PASSWD"]
             else:
-                # Shouldn't go here...
                 passwd = gen_strong_pass()
 
             psk = None
@@ -523,8 +609,8 @@ class BurnCommand(PluginCommand):
 
             image = 'latest' or arguments.IMAGE
 
-            environ_DEV = os.environ['DEV'] if 'DEV' in os.environ else None
-            devices = arguments["--device"] or environ_DEV or None
+            dev = os.environ['DEV'] if 'DEV' in os.environ else None
+            devices = arguments["--device"] or dev or None
 
             if devices is not None:
                 devices = Parameter.expand_string(devices)
@@ -535,7 +621,7 @@ class BurnCommand(PluginCommand):
             key = arguments.sshkey
             tag = arguments['--tag']
 
-            if os_is_pi():
+            if os_is_pi() or os_is_linux():
                 blocksize = arguments.blocksize
                 StopWatch.start("total")
 
@@ -546,7 +632,7 @@ class BurnCommand(PluginCommand):
                     blocksize=blocksize,
                     progress=True,
                     hostnames=hostnames,
-                    # not difference between names and name, maybe we shoudl allign
+                    # not difference between names and name, maybe we should align
                     ips=ips,
                     key=key,
                     password=passwd,
@@ -559,7 +645,7 @@ class BurnCommand(PluginCommand):
 
                 StopWatch.benchmark(sysinfo=False, csv=False)
             else:
-                Console.error("This command is only supported ona Pi")
+                Console.error("This command is only supported ona Pi and Linux")
             return ""
 
         Console.error("see manual page: cms help burn")
