@@ -1,30 +1,42 @@
 import os
 import platform
 import sys
+
 from cloudmesh.common.util import readfile
+from cloudmesh.common.console import Console
 
-import requests
+import hashlib
+# import mmap
 
-# noinspection PyPep8
-if False:
-    ##############################################
-    #
-    # Ignore on pi:  DH KEY TOO SMALL
-    #
-    # see: https://stackoverflow.com/questions/38015537/python-requests-exceptions-sslerror-dh-key-too-small
-    #
+BUF_SIZE = 65536
 
-    requests.packages.urllib3.disable_warnings()
-    requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS = 'AES128-SHA'
-    try:
-        # noinspection PyPep8
-        requests.packages.urllib3.contrib.pyopenssl.DEFAULT_SSL_CIPHER_LIST = 'AES128-SHA'
-    except AttributeError:
-        # no pyopenssl support used / needed / available
-        pass
 
-    #
-    ##############################################
+def sha1sum(filename):
+    Console.info("Verifying sha1")
+    h = hashlib.sha1()
+    with open(filename, 'rb') as f:
+        # with mmap.mmap(f.fileno(), 0, prot=mmap.PROT_READ) as mm:
+        #     h.update(mm)
+        while True:
+            data = f.read(BUF_SIZE)
+            if not data:
+                break
+            h.update(data)
+    return h.hexdigest()
+
+
+def sha256sum(filename):
+    Console.info("Verifying sha256")
+    h = hashlib.sha256()
+    with open(filename, 'rb') as f:
+        # with mmap.mmap(f.fileno(), 0, prot=mmap.PROT_READ) as mm:
+        #     h.update(mm)
+        while True:
+            data = f.read(BUF_SIZE)
+            if not data:
+                break
+            h.update(data)
+    return h.hexdigest()
 
 
 def os_is_windows():
@@ -79,7 +91,7 @@ def check_root(dryrun=False, terminate=True):
     """
     check if I am the root user. If not simply exist the program.
 
-    TODO: should probably mocve to cloudmesh.common
+    TODO: should probably move to cloudmesh.common
 
     :param dryrun: if set to true, does not terminate if not root user
     :type dryrun: bool
